package ij.process;
import ij.measure.Calibration;

/** 16-bit image statistics, including histogram. */
public class ShortStatistics extends ImageStatistics {

	protected boolean unsigned;

	/** Construct an ImageStatistics object from a ShortProcessor
		using the standard measurement options (area, mean,
		mode, min and max). */
	public ShortStatistics(ImageProcessor ip) {
		this(ip, AREA+MEAN+MODE+MIN_MAX, null);
	}

	/** Constructs a ShortStatistics object from a ShortProcessor using
		the specified measurement options. The 'cal' argument, which
		can be null, is currently ignored. */
	public ShortStatistics(ImageProcessor ip, int mOptions, Calibration cal) {
		this.width = ip.getWidth();
		this.height = ip.getHeight();
		unsigned = ip.isUnsigned();
		setup(ip, cal);
		nBins = 256;
		getStatistics(ip);
		if ((mOptions&MODE)!=0)
			getMode();
		if ((mOptions&CENTROID)!=0)
			getCentroid(ip);
		if ((mOptions&CENTER_OF_MASS)!=0)
			getCenterOfMass(ip);
	}

	public void getStatistics(ImageProcessor ip) {
		int v;
		short[] pixels = (short[])ip.getPixels();
		histogram = new int[nBins];
		double sum = 0;
		double sum2 = 0;
		int[] mask = ip.getMask();
		
		// Find min and max
		int roiMin = Integer.MAX_VALUE;
		int roiMax = Integer.MIN_VALUE;
		for (int y=ry, my=0; y<(ry+rh); y++, my++) {
			int i = y * width + rx;
			int mi = my * rw;
			for (int x=rx; x<(rx+rw); x++) {
				if (mask==null||mask[mi++]==ip.BLACK) {
					v = pixels[i];
					if (v<roiMin) roiMin = v;
					if (v>roiMax) roiMax = v;
				}
				i++;
			}
		}
		min = roiMin; max = roiMax;
		histMin = roiMin; histMax = roiMax;

		// Generate histogram
		double scale = (float)((nBins-1)/(histMax-histMin));
		int hMin = (int)histMin;
		binSize = (histMax-histMin)/nBins;
		pixelCount = 0;
		for (int y=ry, my=0; y<(ry+rh); y++, my++) {
			int i = y * width + rx;
			int mi = my * rw;
			for (int x=rx; x<(rx+rw); x++) {
				if (mask==null||mask[mi++]==ip.BLACK) {
					v = pixels[i];
					pixelCount++;
					sum += v;
					sum2 += v*v;
					histogram[(int)(scale*(v-hMin))]++;
				}
				i++;
			}
		}

		pixelCount = rw*rh;
		area = pixelCount*pw*ph;
		mean = sum/pixelCount;
		if (unsigned)
			{min+=32768; max+=32768; histMin+=32768; histMax+=32768; mean+=32768;}
		calculateStdDev(pixelCount, sum, sum2);
	}


	void getMode() {
        int count;
        maxCount = 0;
        for (int i = 0; i < nBins; i++) {
        	count = histogram[i];
            if (count > maxCount) {
                maxCount = count;
                mode = i;
            }
        }
        dmode = histMin+mode*((histMax-histMin)/nBins);
	}

	void getCenterOfMass(ImageProcessor ip) {
		short[] pixels = (short[])ip.getPixels();
		int[] mask = ip.getMask();
		boolean unsigned = ip.isUnsigned();
		int i, mi;
		double v, count=0.0, xsum=0.0, ysum=0.0;
		for (int y=ry,my=0; y<(ry+rh); y++,my++) {
			i = y*width + rx;
			mi = my*rw;
			for (int x=rx; x<(rx+rw); x++) {
				if (mask==null || mask[mi++]==ip.BLACK) {
					v = unsigned?pixels[i]+32768:pixels[i];
					count += v;
					xsum += x*v;
					ysum += y*v;
				}
				i++;
			}
		}
		xCenterOfMass = (xsum/count+0.5)*pw;
		yCenterOfMass = (ysum/count+0.5)*ph;
	}

}